#!/bin/bash -p
# 
# Copy the appropriate files, in the correct order,
# to the boot partition on the SD card.
# Then, if requested, copy the rootfs to its partition.
#
# Order of install:
# Raspberry Pi firmware
# kernel.img - Linux Kernel
# special cmdline.txt - Linux kernel configuration options
# special config.txt - Raspberry Pi bootloader configuration options
# ramdisk - Root file system as ext3 filesystem image
# ------------------------------------------------

#--------------------------------------------------------------
# Initialization
#--------------------------------------------------------------
BOOTDIR=none
ROOTFSDIR=none
SDROOTFS=/media/bbrootfs
FIRMWARE=firmware
IMGFILE=
DOFILE=0
DOBOOTSCR=0
DOUENV=0

# These file names are configurable
UIMAGE="kernel.img"
DESTUIMAGE="$UIMAGE"
RAMDISK="rootfs.ext3"
BOOTSCR="$FIRMWARE/cmdline.txt"
UENV="$FIRMWARE/config.txt"

#--------------------------------------------------------------
# Provide command line usage assistance
#--------------------------------------------------------------
function doHelp
{
    echo ""
    echo "$0 [-b <dir> | -d <device> | -l <file> | -s <file> | -t <file> | -r <file> | -f <file>]"
    echo "where"
    echo "-b <dir>     Directory where boot partition is mounted; Default: $BOOTDIR"
    echo "-d <device>  Device (such as /dev/sdb2) for root file system; Default: $ROOTFSDIR"
    echo "-l <file>    Image file to use for Linux kernel; Default: $UIMAGE"
    echo "-s <file>    Boot script file; Default: $BOOTSCR"
    echo "-t <file>    Text boot script; Default: $UENV"
    echo "-r <file>    ext3 image for root file system; Default: $RAMDISK"
    echo "-f <file>    Image file to use instead of a device"
    echo ""
    echo "If -b is specified then the boot files will be installed"
    echo "If -d is specified then the ramdisk will be installed"
    echo "Default is to do nothing."
    echo ""
}

# Setup to use an image file as the device
function doFile
{
    echo "doFile: IMGFILE = $IMGFILE "

    if [ ! -f $IMGFILE ]
    then
        echo "No such file: $IMGFILE"
        exit 1
    fi

    CNT=1
    sudo kpartx -a $IMGFILE
    sudo kpartx -l $IMGFILE > $0.$$
    while read line
    do
        partition=`echo $line | cut -f1 -d" "`
        case $CNT in
        1)  mkdir -p bootdir
            sudo mount /dev/mapper/$partition bootdir;;
        2)  ROOTFSDIR=/dev/mapper/$partition;;
        esac
        CNT=$((CNT + 1))
    done < $0.$$
    BOOTDIR=bootdir
    echo "doFile: BOOTDIR = $BOOTDIR"
    echo "doFile: ROOTFSDIR = $ROOTFSDIR"
    sudo rm -f $0.$$
}

# Cleanup from use of an image file as the device
function doUnfile
{
    sudo umount bootdir
    sudo kpartx -d $IMGFILE
    rmdir bootdir
}

#--------------------------------------------------------------
# Read command line arguments
#--------------------------------------------------------------
while getopts ":f:F:b:r:u:l:s:t:d:" Option
do
    case $Option in
    b) BOOTDIR=$OPTARG;;
    d) ROOTFSDIR=$OPTARG;;
    l) UIMAGE=$OPTARG;;
    s) BOOTSCR=$OPTARG
       DOBOOTSCR=1
       ;;
    t) UENV=$OPTARG
       DOUENV=1
       ;;
    r) RAMDISK=$OPTARG;;
    f) DOFILE=1
       IMGFILE=$OPTARG;;
    F) IMGFILE=$OPTARG
       doUnfile
       exit 0;;
    *) doHelp; exit 0;;
    esac
done

if [ $DOFILE -eq 1 ]
then
    doFile
fi
echo "BOOTDIR = $BOOTDIR"
echo "ROOTFSDIR = $ROOTFSDIR"

#--------------------------------------------------------------
# Error checking
#--------------------------------------------------------------
if [ "$BOOTDIR" != "none" ]
then
    if [ ! -d $BOOTDIR ]; then
        echo "The boot directory ($BOOTDIR) is not mounted.  Installation aborted."
        doHelp
        exit 1
    fi
    if [ ! -f $UIMAGE ]; then
        echo "Your missing the Linux kernel binary.  Install aborted."
        doHelp
        exit 1
    fi
    if [ ! -d $FIRMWARE ]; then
        echo "Your missing the $FIRMWARE directory.  Install aborted."
        doHelp
        exit 1
    fi
    if [ ! -f $UENV ]; then
        echo "Your missing the uEnv.txt script.  Install aborted."
        doHelp
        exit 1
    fi
    if [ ! -f $BOOT ]; then
        echo "Your missing the boot.scr boot script.  Install aborted."
        doHelp
        exit 1
    fi
else
    echo "No boot directory specified. Boot files will not be installed."
fi

if [ "$ROOTFSDIR" != "none" ]
then
    if [ ! -b $ROOTFSDIR ]; then
        echo "The rootfs device ($ROOTFSDIR) is not available.  Installation aborted."
        doHelp
        exit 1
    fi
    if [ ! -f $RAMDISK ]; then
        echo "Your missing the rootfs image ($RAMDISK).  Install aborted."
        doHelp
        exit 1
    fi
else
    echo "No root file system device specified. Ramdisk image will not be installed."
fi

if [ "$BOOTDIR" = "none" ] && [ "$ROOTFSDIR" = "none" ]
then
    doHelp
    exit 1
fi

#--------------------------------------------------------------
# Install firmware, kernel and configurations
#--------------------------------------------------------------
if [ "$BOOTDIR" != "none" ]
then
    # First, copy in all the firmware
    sudo cp $FIRMWARE/* $BOOTDIR/

    # Now remove the upstream kernel images - we have our own.
    sudo rm $BOOTDIR/kernel*

    # Copy in our kernel.
    sudo cp $UIMAGE $BOOTDIR/$DESTUIMAGE

    # Do we have special config.txt and cmdline.txt files to copy in?
    if [ $DOBOOTSCR -eq 1 ]
    then
        sudo cp $BOOTSCR $BOOTDIR
    fi
    if [ $DOUENV -eq 1 ]
    then
        sudo cp $UENV $BOOTDIR
    fi
fi

#--------------------------------------------------------------
# Install ramdisk image
#--------------------------------------------------------------
if [ "$ROOTFSDIR" != "none" ]
then
    # Copy image to partition
    sudo dd if=$RAMDISK of=$ROOTFSDIR bs=16384
    sudo e2label $ROOTFSDIR rootfs
    sudo e2fsck -f $ROOTFSDIR
    sudo resize2fs $ROOTFSDIR

    # Set permissions on partitions files/directories
    sudo mkdir -p $SDROOTFS
    sudo mount $ROOTFSDIR $SDROOTFS
    cwd=`pwd`
    cd $SDROOTFS && sudo chown -R root.root .
    cd $cwd
    sudo umount $SDROOTFS
    sudo rmdir $SDROOTFS
fi

#--------------------------------------------------------------
# Cleanup image file setup
#--------------------------------------------------------------
if [ $DOFILE -eq 1 ]
then
    doUnfile
fi
